@@ -1,6 +1,6 @@
 /*
   This file is part of Challenger
-  (C) 2023 Taler Systems SA
+  (C) 2023, 2024 Taler Systems SA
 
   Challenger is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -94,6 +94,16 @@ struct GNUNET_TIME_Relative CH_validation_expiration;
  */
 struct GNUNET_TIME_Relative CH_pin_retransmission_frequency;
 
+/**
+ * JSON object with key-object pairs mapping address keys (from the
+ * form) to an object with a field "regex" containing a regular
+ * expressions expressing restrictions on values for the address and a
+ * field "hint" (and possibly "hint_i18n") containing a human-readable
+ * message explaining the restriction. Missing map entries indicate
+ * that the input is unrestricted.
+ */
+json_t *CH_restrictions;
+
 /**
  * Type of addresses this challenger validates.
  */
@@ -377,14 +387,18 @@ handle_mhd_completion_callback (void *cls,
                                 enum MHD_RequestTerminationCode toe)
 {
   struct CH_HandlerContext *hc = *con_cls;
+  const union MHD_ConnectionInfo *ci;
 
   (void) cls;
   if (NULL == hc)
     return;
   GNUNET_assert (hc->connection == connection);
+  ci = MHD_get_connection_info (connection,
+                                MHD_CONNECTION_INFO_HTTP_STATUS);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Finished handling request with status %d\n",
-              (int) toe);
+              "Finished handling request with status %d (HTTP status %u)\n",
+              (int) toe,
+              ci->http_status);
   if (NULL != hc->cc)
     hc->cc (hc->ctx);
   GNUNET_free (hc);
@@ -589,6 +603,31 @@ run (void *cls,
                                "ADDRESS_TYPE");
     return;
   }
+  {
+    char *restrictions;
+
+    if (GNUNET_OK ==
+        GNUNET_CONFIGURATION_get_value_string (config,
+                                               "CHALLENGER",
+                                               "ADDRESS_RESTRICTIONS",
+                                               &restrictions))
+    {
+      json_error_t err;
+
+      CH_restrictions = json_loads (restrictions,
+                                    JSON_REJECT_DUPLICATES,
+                                    &err);
+      GNUNET_free (restrictions);
+      if (NULL == CH_restrictions)
+      {
+        GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
+                                   "CHALLENGER",
+                                   "ADDRESS_RESTRICTIONS",
+                                   err.text);
+        return;
+      }
+    }
+  }
 
   global_ret = EXIT_NOTCONFIGURED;
   GNUNET_SCHEDULER_add_shutdown (&do_shutdown,
